import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import * as React from 'react';
import * as immutable from 'object-path-immutable';
import Column from '../../layout/Column';
import TextField from '../../core/TextField';
import NumberField from '../../core/NumberField';
import PresentationFormHelperText from './PresentationFormHelperText';
import getLocalUploads from './getLocalUploads';
import { getErrorAtPath, isPathEqual, getPropertyAtPath } from './utilities';
// Inputs
import ArrayInput from './inputs/ArrayInput';
import BooleanInput from './inputs/BooleanInput';
import DateInput from './inputs/DateInput';
import FacebookAuthInput from './inputs/FacebookAuthInput';
import FileInput from './inputs/FileInput';
import GoogleAuthInput from './inputs/GoogleAuthInput';
import GoogleFilePickerInput from './inputs/GoogleFilePickerInput';
import ImagePickerFieldInput from './inputs/ImagePickerFieldInput';
import ModalButton from './inputs/ModalButton';
import NumberInput from './inputs/NumberInput';
import OAuthInput from './inputs/OAuthInput';
import OAuthInput2 from './inputs/OAuthInput2';
import OneDriveAuthInput from './inputs/OneDriveAuthInput';
import PlaylistInput from './inputs/PlaylistInput';
import PlaylistInputLegacy from './inputs/PlaylistInput/PlaylistInputLegacy';
import PosterMyWallAuthInput from './inputs/PosterMyWallAuthInput';
import CatalogInput from './inputs/CatalogInput';
import SelectionInput from './inputs/SelectionInput';
import SoundZoneInput from './inputs/SoundZoneInput';
import StringInput from './inputs/StringInput';
import TextInput from './inputs/TextInput';
import ThemeInput from './inputs/ThemeInput';
import ThemeInputLegacy from './inputs/ThemeInput/ThemeInputLegacy';
import ToggleButtonGroupInput from './inputs/ToggleButtonGroupInput';
// These are "fake" presentation properties for name and duration so they can
// be treated the same as application variables when updating the presentation state.
const namePath = ['name'];
const durationPath = ['duration'];
const nameProp = { type: 'string', name: 'name' };
const durationProp = { type: 'number', name: 'duration' };
const PresentationForm = ({ className, presentation, appVersion, errors, themes, soundZones, playlists, catalogs, selectedPlaylistPath, builderState, newThemeDisabled, newThemeDisabledMessage, onChange, onBuilderStateChange, onPlaylistEdit, onPlaylistCreate, onPlaylistSelect, onThemeEdit, onThemeManage, onThemeAdd, themeOptions, createObjectURL, }) => {
    // Refs
    const fileBlobsRef = React.useRef({});
    // Callbacks
    const updatePresentation = React.useCallback((path, value, property, file) => {
        // Remove the value if null or undefined for array inputs.
        const shouldDelete = property.type === 'array' && (value === null || value === undefined);
        const updatedPresentation = shouldDelete
            ? immutable.del(presentation, path)
            : immutable.set(presentation, path, value);
        if (file && value) {
            fileBlobsRef.current[value.url] = file;
        }
        onChange(updatedPresentation, getLocalUploads(updatedPresentation, appVersion, fileBlobsRef.current));
    }, [presentation, appVersion, onChange]);
    const setInputState = React.useCallback((path, state) => {
        if (!onBuilderStateChange)
            return;
        // @ts-ignore
        const inputFound = builderState.inputs.find((i) => isPathEqual(i.path, path));
        const updatedInputs = inputFound
            ? // @ts-ignore
                builderState.inputs.map((i) => isPathEqual(i.path, path)
                    ? { path, state: Object.assign(Object.assign({}, inputFound.state), state) }
                    : i)
            : // @ts-ignore
                [...builderState.inputs, { path, state }];
        onBuilderStateChange(Object.assign(Object.assign({}, builderState), { inputs: updatedInputs }));
    }, [onBuilderStateChange, builderState]);
    const getInputStateAtPath = React.useCallback((path) => {
        // @ts-ignore
        const input = builderState.inputs.find((i) => {
            return isPathEqual(path, i.path);
        });
        return input && input.state;
    }, [builderState]);
    // Effects
    // Check hash params for updated properties.
    React.useEffect(() => {
        // Get the window hash without the leading '#'.
        const hash = window.location.hash.substr(1);
        // Parse hash params by splitting on & (or #) and then =.
        //   ie. #applicationVariables.headingText=text => [['applicationVariables.headingText', 'SomeText'], ...]
        const hashParams = hash
            .split(/[&#]/)
            .map((paramStr) => paramStr.split('='));
        hashParams.forEach((hashParam) => {
            if (hashParam.length !== 2)
                return;
            const path = hashParam[0]
                .split('.')
                .filter((pathPart) => !!pathPart);
            if (!path.length)
                return;
            // Try to find the property at this path.
            let property;
            if (path[0] === 'applicationVariables') {
                property = getPropertyAtPath(appVersion.presentationProperties, path);
            }
            else if (isPathEqual(path, namePath)) {
                property = nameProp;
            }
            else if (isPathEqual(path, durationPath)) {
                property = durationProp;
            }
            // @ts-ignore
            if (!property)
                return;
            let value = decodeURIComponent(hashParam[1]);
            try {
                value = JSON.parse(value);
            }
            catch (err) {
                // If we can't parse valid JSON, assume a string value is passed in.
                // This allows the hash param to omit quotes (ie. #headingText=text)
            }
            if (value !== undefined) {
                updatePresentation(path, value, property, undefined);
            }
        });
        // Clear hash parameters to avoid leaking tokens and other sensitive
        // values to the user.
        window.location.hash = '';
    }, [appVersion, updatePresentation]);
    // Render
    const nameError = getErrorAtPath(errors, namePath);
    const durationError = getErrorAtPath(errors, durationPath);
    const renderApplicationVariables = (appVars, properties, path, strings) => {
        const propertyTypeIndexes = {};
        const formInputs = properties.map((property) => {
            const propertyPath = [...path, property.name];
            const propertyTypeIndex = propertyTypeIndexes[property.type] || 0;
            propertyTypeIndexes[property.type] = propertyTypeIndex + 1;
            return renderInput(property, propertyTypeIndex, 
            // appVars can be undefined for newly added array items.
            appVars && appVars[property.name], appVars, propertyPath, strings);
        });
        return formInputs;
    };
    const renderInput = (property, propertyTypeIndex, value = property.default, parentValue, path, strings) => {
        const key = property.name;
        const label = strings[property.name] || property.name;
        const constraints = property.constraints || {};
        const isDisabled = !!property.disable;
        const inputError = getErrorAtPath(errors, path);
        const hasError = !!inputError;
        // Render the component inline by calling the function instead of using JSX.
        // This causes `null` to be returned instead of an empty <PresentationFormHelperText />
        // element. Some inputs (ie. PlaylistInput) provide their own default helperText if not set
        // for the input and rely `null` being returned here.
        const helperText = PresentationFormHelperText({
            property,
            path,
            strings,
            error: inputError,
        });
        if (property.hide) {
            return null;
        }
        switch (property.type) {
            case 'string': {
                return (_jsx(StringInput, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'text': {
                return (_jsx(TextInput, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'number': {
                return (_jsx(NumberInput, { label: label, value: value, defaultValue: property.default, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'boolean': {
                return (_jsx(BooleanInput, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'selection': {
                return (_jsx(SelectionInput, { label: label, value: value, parentValue: parentValue, multiple: property.multiple, searchable: property.searchable, selectable: property.selectable, sortable: property.sortable, options: property.options, optionsUrl: property.options_url, optionsUrlHttpHeaders: property.options_url_http_headers, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property), strings: strings, inputState: getInputStateAtPath(path), onInputStateChange: (state) => {
                        setInputState(path, state);
                    } }, key));
            }
            case 'toggleButtonGroup': {
                return (_jsx(ToggleButtonGroupInput, { label: label, value: value, options: property.options, helperText: helperText, disabled: isDisabled, onChange: (newValue) => {
                        // Enforce at least one button to be active
                        if (newValue !== null) {
                            updatePresentation(path, newValue, property);
                        }
                    }, exclusive: property.exclusive, strings: strings }, key));
            }
            case 'selectionWithImages': {
                return (_jsx(ImagePickerFieldInput, { parentValue: parentValue, value: value, imagesUrl: property.images_url, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'date': {
                return (_jsx(DateInput, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'file': {
                return (_jsx(FileInput, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, fileType: property.file_type, constraints: constraints, onChange: (newValue, file) => 
                    // @ts-ignore
                    updatePresentation(path, newValue, property, file), createObjectURL: createObjectURL }, key));
            }
            case 'oAuth2': {
                return (_jsx(OAuthInput2, { path: path, value: value, provider: property.provider || '', iconUrl: property.icon_url || '', authUrl: property.auth_url || '', verifyUrl: property.verify_url || '', verifyQsParam: property.verify_qs_param || '', logoutUrl: property.logout_url || '', logoutQsParam: property.logout_qs_param || '', backgroundColor: property.background_color, foregroundColor: property.foreground_color, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'oAuth': {
                return (_jsx(OAuthInput, { path: path, label: label, value: value, provider: property.provider, authUrl: property.auth_url || '', verifyUrl: property.verify_url || '', verifyQsParam: property.verify_qs_param || '', helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'facebookAuth': {
                return (
                // @ts-ignore
                _jsx(FacebookAuthInput, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'googleAuth': {
                return (
                // @ts-ignore
                _jsx(GoogleAuthInput, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'onedriveAuth': {
                return (
                // @ts-ignore
                _jsx(OneDriveAuthInput, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'postermywallAuth': {
                return (
                // @ts-ignore
                _jsx(PosterMyWallAuthInput, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'googleFilePicker': {
                return (_jsx(GoogleFilePickerInput, { label: label, disabled: isDisabled, helperText: helperText, error: hasError, picker: {
                        developerKey: property.developerKey,
                        accessToken: property.accessToken || '',
                        accessTokenUrl: property.accessTokenUrl || '',
                        presentationValues: parentValue || {},
                        title: property.title,
                        views: property.views,
                        onChangeCallback: (newValue) => {
                            updatePresentation(path, newValue, property);
                        },
                    } }, key));
            }
            case 'theme': {
                if (!themeOptions) {
                    // This is to only here to support the RaydiantKit Simulator
                    return (_jsx(ThemeInputLegacy, { label: label, 
                        // @ts-ignore
                        value: presentation.themeId, 
                        // @ts-ignore
                        themes: themes, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(['themeId'], newValue, property) }, key));
                }
                return (_jsx(ThemeInput, { label: label, 
                    // @ts-ignore
                    value: presentation.themeId, themeOptions: themeOptions, helperText: helperText, error: hasError, disabled: isDisabled, addDisabled: newThemeDisabled, addDisabledMessage: newThemeDisabledMessage, onChange: (newValue) => updatePresentation(['themeId'], newValue, property), onEdit: onThemeEdit, onManage: onThemeManage, 
                    // @ts-ignore
                    onAdd: onThemeAdd }, key));
            }
            case 'playlist': {
                if (!onPlaylistEdit) {
                    // If onPlaylistEdit is not provided then use the legacy playlist input.
                    // This is to only here to support the RaydiantKit Simulator
                    return (_jsx(PlaylistInputLegacy, { label: label, value: value, 
                        // @ts-ignore
                        playlists: playlists, helperText: helperText, error: hasError, disabled: isDisabled, propertyTypeIndex: propertyTypeIndex, onChange: (newValue) => updatePresentation(path, newValue, property), 
                        // @ts-ignore
                        onEdit: (playlistId) => onPlaylistEdit(playlistId, path), 
                        // @ts-ignore
                        onCreate: () => onPlaylistCreate(path) }, key));
                }
                return (_jsx(PlaylistInput, { label: label, value: value, path: path, 
                    // @ts-ignore
                    playlists: playlists, helperText: helperText, error: hasError, disabled: isDisabled, propertyTypeIndex: propertyTypeIndex, 
                    // @ts-ignore
                    selectedPlaylistPath: selectedPlaylistPath, onChange: (newValue) => updatePresentation(path, newValue, property), onEdit: (playlistId) => onPlaylistEdit(playlistId, path), 
                    // @ts-ignore
                    onSelect: () => onPlaylistSelect(path) }, key));
            }
            case 'soundZone': {
                return (_jsx(SoundZoneInput, { label: label, value: value, 
                    // @ts-ignore
                    soundZones: soundZones, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'array': {
                const singularLabel = (property.singular_name && strings[property.singular_name]) ||
                    property.singular_name;
                return (_jsx(ArrayInput, { path: path, label: label, value: value, 
                    // @ts-ignore
                    singularLabel: singularLabel, 
                    // @ts-ignore
                    properties: property.properties, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, strings: strings, errors: errors, onChange: (newValue) => updatePresentation(path, newValue, property), renderForm: (itemValue, itemProperties, itemPath) => (_jsx(Column, { children: renderApplicationVariables(itemValue, itemProperties, [...path, ...itemPath], strings) }, void 0)), onInputStateChange: (state) => {
                        setInputState(path, state);
                    } }, key));
            }
            case 'modal': {
                return (_jsx(ModalButton, { label: label, sourceUrl: property.sourceUrl, backgroundColor: property.backgroundColor, hoveredBackgroundColor: property.hoveredBackgroundColor, textColor: property.textColor, parentValue: parentValue, helperText: helperText, disabled: isDisabled, onChange: (newValue) => 
                    // TODO: This will save the state to the db, we used to use updateIgnoredApplicationVariables
                    // to bypass the db but this seems hacky, find a better way.
                    // updateIgnoredApplicationVariables(path, newValue)
                    updatePresentation(path, newValue, property) }, key));
            }
            case 'catalog': {
                return (_jsx(CatalogInput, { label: label, value: value, disabled: isDisabled, helperText: helperText, catalogs: catalogs, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            default:
                return null;
        }
    };
    return (_jsxs(Column, Object.assign({ className: className }, { children: [_jsx(TextField, { label: "name", value: presentation.name, onChange: (name) => updatePresentation(namePath, name, nameProp), helperText: _jsx(PresentationFormHelperText, { property: nameProp, path: namePath, error: nameError, strings: appVersion.strings }, void 0), error: !!nameError }, void 0), renderApplicationVariables(presentation.applicationVariables, appVersion.presentationProperties, ['applicationVariables'], appVersion.strings), appVersion.configurableDuration && (_jsx(NumberField, { label: "Duration", value: presentation.duration, onChange: (duration) => updatePresentation(durationPath, duration, durationProp), helperText: _jsx(PresentationFormHelperText, { property: durationProp, path: durationPath, error: durationError, strings: appVersion.strings }, void 0), error: !!durationError }, void 0))] }), void 0));
};
export default PresentationForm;
//# sourceMappingURL=PresentationForm.js.map