import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import * as React from 'react';
import cn from 'classnames';
import debounce from 'debounce';
import Tabs from '../../core/Tabs/Tabs';
import Hidden from '../../layout/Hidden';
import { PreviewMode, PreviewResolutions, } from '../PresentationTypes';
import useStyles from './PresentationPreview.styles';
import SelectField from '../../core/SelectField';
import { applyScaling } from '../../utils/scaleUtils';
const PresentationPreview = ({ appVersion, previewMode, previewResolution, showBorder, children, onPreviewModeChange, enableResolutionSelection, is4kEnabled = false, is8kEnabled = false, onPreviewResolutionChange, }) => {
    const previewBorderWidth = applyScaling(previewResolution);
    const classes = useStyles(previewBorderWidth)();
    // Refs
    const previewRef = React.useRef(null);
    // State
    const [dimensions, setDimensions] = React.useState({
        width: 0,
        height: 0,
    });
    // Callbacks
    const calculateDimensions = React.useCallback(() => {
        if (!previewRef.current)
            return;
        const rect = previewRef.current.getBoundingClientRect();
        const previewWidth = Math.ceil(rect.width);
        const previewHeight = Math.ceil(rect.height);
        if (previewWidth !== dimensions.width ||
            previewHeight !== dimensions.height) {
            setDimensions({ width: previewWidth, height: previewHeight });
        }
    }, [dimensions, setDimensions]);
    // Effects
    // Calculate initial dimensions
    React.useLayoutEffect(calculateDimensions);
    // Re-calculate on window resize
    React.useEffect(() => {
        const calculateDebounced = debounce(calculateDimensions);
        window.addEventListener('resize', calculateDebounced);
        window.addEventListener('orientationchange', calculateDebounced);
        return () => {
            window.removeEventListener('resize', calculateDebounced);
            window.removeEventListener('orientationchange', calculateDebounced);
        };
    }, [calculateDimensions]);
    // Render
    // pixels
    const getResolution = () => {
        switch (previewResolution) {
            case PreviewResolutions.HD:
                return { width: 1280, height: 720 };
            case PreviewResolutions.UltraHD:
                return { width: 3840, height: 2160 };
            case PreviewResolutions.EightK:
                return { width: 7680, height: 4320 };
            default:
                return { width: 1920, height: 1080 };
        }
    };
    const isLandscape = previewMode !== PreviewMode.Vertical;
    const resolution = getResolution();
    // To ensure the preview content is 1920x1080 we need to compensate for
    // the border width around the preview.
    const frameSize = showBorder ? previewBorderWidth * 2 : 0;
    const width = (isLandscape ? resolution.width : resolution.height) + frameSize;
    const height = (isLandscape ? resolution.height : resolution.width) + frameSize;
    const scaleX = dimensions.width / width;
    const scaleY = dimensions.height / height;
    const scale = Math.min(scaleX, scaleY);
    const marginTop = -(height / 2) * scale;
    const marginLeft = -(width / 2) * scale;
    return (_jsxs("div", Object.assign({ className: classes.root }, { children: [_jsx("div", Object.assign({ className: classes.main }, { children: _jsx("div", Object.assign({ ref: previewRef, className: classes.previewContainer }, { children: _jsx("div", Object.assign({ className: cn(classes.preview, showBorder && classes.previewBorder), style: {
                            width,
                            height,
                            marginTop,
                            marginLeft,
                            transform: `scale(${scale})`,
                        } }, { children: children }), void 0) }), void 0) }), void 0), _jsxs("div", Object.assign({ className: classes.footer }, { children: [appVersion && (_jsxs("div", Object.assign({ className: classes.app }, { children: [_jsx("div", { className: classes.appIcon, style: {
                                    backgroundImage: appVersion.iconUrl
                                        ? `url(${appVersion.iconUrl})`
                                        : '',
                                } }, void 0), appVersion.name && (_jsxs("div", { children: [_jsx("div", Object.assign({ className: classes.appName }, { children: appVersion.name }), void 0), _jsx("div", Object.assign({ className: classes.appDescription }, { children: appVersion.description || appVersion.strings.description }), void 0)] }, void 0))] }), void 0)), !appVersion && onPreviewModeChange && (_jsxs("div", Object.assign({ className: classes.app }, { children: [_jsx("div", { className: classes.appIcon, style: { backgroundColor: 'transparent' } }, void 0), _jsxs("div", { children: [_jsx("div", { className: classes.appName }, void 0), _jsx("div", { className: classes.appDescription }, void 0)] }, void 0)] }), void 0)), onPreviewModeChange && (_jsx(Hidden, Object.assign({ smDown: true }, { children: _jsxs(Tabs, Object.assign({ inline: true }, { children: [enableResolutionSelection && onPreviewResolutionChange && (_jsxs(SelectField, Object.assign({ label: "Display Resolution", value: previewResolution || PreviewResolutions.FullHD, onChange: (resolution) => onPreviewResolutionChange(resolution) }, { children: [_jsx("option", Object.assign({ value: PreviewResolutions.HD, className: classes.resolutionSelectOption }, { children: "720p" }), void 0), _jsx("option", Object.assign({ value: PreviewResolutions.FullHD, className: classes.resolutionSelectOption }, { children: "1080p" }), void 0), is4kEnabled && (_jsx("option", Object.assign({ value: PreviewResolutions.UltraHD, className: classes.resolutionSelectOption }, { children: "4K" }), void 0)), is8kEnabled && (_jsx("option", Object.assign({ value: PreviewResolutions.EightK, className: classes.resolutionSelectOption }, { children: "8K" }), void 0))] }), void 0)), _jsx(Tabs.Item, { icon: "horizontalScreen", label: "Landscape", active: previewMode === PreviewMode.Horizontal, onClick: () => onPreviewModeChange(PreviewMode.Horizontal) }, void 0), _jsx(Tabs.Item, { icon: "verticalScreen", label: "Portrait", active: previewMode === PreviewMode.Vertical, onClick: () => onPreviewModeChange(PreviewMode.Vertical) }, void 0)] }), void 0) }), void 0))] }), void 0)] }), void 0));
};
export default PresentationPreview;
//# sourceMappingURL=PresentationPreview.js.map